@@ -7,7 +7,7 @@ var searchData=
   ['libinput_5fconfig_5fdwt_5fstate',['libinput_config_dwt_state',['../group__config.html#gacf6846753a607f830ebbd0d3d181f813',1,'libinput.h']]],
   ['libinput_5fconfig_5fmiddle_5femulation_5fstate',['libinput_config_middle_emulation_state',['../group__config.html#ga43d9b6271fdbf41b3a0591dea9d1b86a',1,'libinput.h']]],
   ['libinput_5fconfig_5fscroll_5fmethod',['libinput_config_scroll_method',['../group__config.html#ga6466e6b9c55e6835a05c7c29c8b7a844',1,'libinput.h']]],
-  ['libinput_5fconfig_5fsend_5fevents_5fmode',['libinput_config_send_events_mode',['../libinput_8h.html#a56c9886b0161ecdb9b351d8dedc212e4',1,'libinput.h']]],
+  ['libinput_5fconfig_5fsend_5fevents_5fmode',['libinput_config_send_events_mode',['../group__config.html#ga56c9886b0161ecdb9b351d8dedc212e4',1,'libinput.h']]],
   ['libinput_5fconfig_5fstatus',['libinput_config_status',['../group__config.html#ga33ff306ed8505129adce4c092a3ed175',1,'libinput.h']]],
   ['libinput_5fconfig_5ftap_5fstate',['libinput_config_tap_state',['../group__config.html#gaca4a3177736e6fd024cb77bc684857a1',1,'libinput.h']]],
   ['libinput_5fdevice_5fcapability',['libinput_device_capability',['../group__device.html#ga3110cdddce94a1df0b8a3306909c8f15',1,'libinput.h']]],
