@@ -32,8 +32,8 @@
 
 #include "evdev-mt-touchpad.h"
 
-#define DEFAULT_BUTTON_ENTER_TIMEOUT 100 /* ms */
-#define DEFAULT_BUTTON_LEAVE_TIMEOUT 300 /* ms */
+#define DEFAULT_BUTTON_ENTER_TIMEOUT ms2us(100)
+#define DEFAULT_BUTTON_LEAVE_TIMEOUT ms2us(300)
 
 /*****************************************
  * BEFORE YOU EDIT THIS FILE, look at the state diagram in
@@ -46,8 +46,6 @@
  * The state machine only affects the soft button area code.
  */
 
-#define CASE_RETURN_STRING(a) case a: return #a;
-
 static inline const char*
 button_state_to_str(enum button_state state) {
 	switch(state) {
@@ -525,20 +523,19 @@ tp_init_softbuttons(struct tp_dispatch *tp,
 	absinfo_y = device->abs.absinfo_y;
 
 	xoffset = absinfo_x->minimum,
-	yoffset = absinfo_y->minimum;
+	yoffset = absinfo_y->minimum,
 	yres = absinfo_y->resolution;
 	width = device->abs.dimensions.x;
 	height = device->abs.dimensions.y;
 
-	/* button height: 10mm or 15% of the touchpad height,
+	/* button height: 10mm or 15% or the touchpad height,
 	   whichever is smaller */
-	if (!device->abs.fake_resolution && (height * 0.15/yres) > 10) {
+	if ((height * 0.15)/yres > 10) {
 		tp->buttons.bottom_area.top_edge =
-		absinfo_y->maximum - 10 * yres;
+			absinfo_y->maximum - 10 * yres;
 	} else {
 		tp->buttons.bottom_area.top_edge = height * .85 + yoffset;
 	}
-
 	tp->buttons.bottom_area.rightbutton_left_edge = width/2 + xoffset;
 }
 
@@ -547,7 +544,7 @@ tp_init_top_softbuttons(struct tp_dispatch *tp,
 			struct evdev_device *device,
 			double topbutton_size_mult)
 {
-	int width, height;
+	int width;
 	const struct input_absinfo *absinfo_x, *absinfo_y;
 	int xoffset, yoffset;
 	int yres;
@@ -559,7 +556,6 @@ tp_init_top_softbuttons(struct tp_dispatch *tp,
 	yoffset = absinfo_y->minimum;
 	yres = absinfo_y->resolution;
 	width = device->abs.dimensions.x;
-	height = device->abs.dimensions.y;
 
 	if (tp->buttons.has_topbuttons) {
 		/* T440s has the top button line 5mm from the top, event
@@ -567,14 +563,8 @@ tp_init_top_softbuttons(struct tp_dispatch *tp,
 		   top - which maps to 15%.  We allow the caller to enlarge the
 		   area using a multiplier for the touchpad disabled case. */
 		double topsize_mm = 10 * topbutton_size_mult;
-		double topsize_pct = .15 * topbutton_size_mult;
 
-		if (!device->abs.fake_resolution) {
-			tp->buttons.top_area.bottom_edge =
-			yoffset + topsize_mm * yres;
-		} else {
-			tp->buttons.top_area.bottom_edge = height * topsize_pct + yoffset;
-		}
+		tp->buttons.top_area.bottom_edge = yoffset + topsize_mm * yres;
 		tp->buttons.top_area.rightbutton_left_edge = width * .58 + xoffset;
 		tp->buttons.top_area.leftbutton_right_edge = width * .42 + xoffset;
 	} else {
@@ -649,22 +639,19 @@ static enum libinput_config_click_method
 tp_click_get_default_method(struct tp_dispatch *tp)
 {
 	struct evdev_device *device = tp->device;
+	uint32_t clickfinger_models = EVDEV_MODEL_CHROMEBOOK |
+				      EVDEV_MODEL_SYSTEM76_BONOBO |
+				      EVDEV_MODEL_SYSTEM76_GALAGO |
+				      EVDEV_MODEL_SYSTEM76_KUDU |
+				      EVDEV_MODEL_CLEVO_W740SU;
 
 	if (!tp->buttons.is_clickpad)
 		return LIBINPUT_CONFIG_CLICK_METHOD_NONE;
 	else if (libevdev_get_id_vendor(tp->device->evdev) == VENDOR_ID_APPLE)
 		return LIBINPUT_CONFIG_CLICK_METHOD_CLICKFINGER;
 
-	switch (device->model) {
-	case EVDEV_MODEL_CHROMEBOOK:
-	case EVDEV_MODEL_SYSTEM76_BONOBO:
-	case EVDEV_MODEL_SYSTEM76_GALAGO:
-	case EVDEV_MODEL_SYSTEM76_KUDU:
-	case EVDEV_MODEL_CLEVO_W740SU:
+	if (device->model_flags & clickfinger_models)
 		return LIBINPUT_CONFIG_CLICK_METHOD_CLICKFINGER;
-	default:
-		break;
-	}
 
 	return LIBINPUT_CONFIG_CLICK_METHOD_BUTTON_AREAS;
 }
@@ -696,7 +683,7 @@ tp_init_middlebutton_emulation(struct tp_dispatch *tp,
 	if (!libevdev_has_event_code(device->evdev, EV_KEY, BTN_MIDDLE)) {
 		enable_by_default = true;
 		want_config_option = false;
-	} else if (device->model == EVDEV_MODEL_ALPS_TOUCHPAD) {
+	} else if (device->model_flags & EVDEV_MODEL_ALPS_TOUCHPAD) {
 		enable_by_default = true;
 		want_config_option = true;
 	} else
@@ -713,8 +700,6 @@ tp_init_buttons(struct tp_dispatch *tp,
 {
 	struct libinput *libinput = tp_libinput_context(tp);
 	struct tp_touch *t;
-	int width, height;
-	double diagonal;
 	const struct input_absinfo *absinfo_x, *absinfo_y;
 
 	tp->buttons.is_clickpad = libevdev_has_property(device->evdev,
@@ -729,7 +714,8 @@ tp_init_buttons(struct tp_dispatch *tp,
 				       "%s: clickpad advertising right button\n",
 				       device->devname);
 	} else if (libevdev_has_event_code(device->evdev, EV_KEY, BTN_LEFT) &&
-		   !tp->buttons.is_clickpad) {
+		   !tp->buttons.is_clickpad &&
+		   libevdev_get_id_vendor(device->evdev) != VENDOR_ID_APPLE) {
 			log_bug_kernel(libinput,
 				       "%s: non clickpad without right button?\n",
 				       device->devname);
@@ -738,19 +724,9 @@ tp_init_buttons(struct tp_dispatch *tp,
 	absinfo_x = device->abs.absinfo_x;
 	absinfo_y = device->abs.absinfo_y;
 
-	/* pinned-finger motion threshold, see tp_unpin_finger.
-	   The MAGIC for resolution-less touchpads ends up as 2% of the diagonal */
-	if (device->abs.fake_resolution) {
-		const double BUTTON_MOTION_MAGIC = 0.007;
-		width = device->abs.dimensions.x;
-		height = device->abs.dimensions.y;
-		diagonal = sqrt(width*width + height*height);
-		tp->buttons.motion_dist.x_scale_coeff = diagonal * BUTTON_MOTION_MAGIC;
-		tp->buttons.motion_dist.y_scale_coeff = diagonal * BUTTON_MOTION_MAGIC;
-	} else {
-		tp->buttons.motion_dist.x_scale_coeff = 1.0/absinfo_x->resolution;
-		tp->buttons.motion_dist.y_scale_coeff = 1.0/absinfo_y->resolution;
-	}
+	/* pinned-finger motion threshold, see tp_unpin_finger. */
+	tp->buttons.motion_dist.x_scale_coeff = 1.0/absinfo_x->resolution;
+	tp->buttons.motion_dist.y_scale_coeff = 1.0/absinfo_y->resolution;
 
 	tp->buttons.config_method.get_methods = tp_button_config_click_get_methods;
 	tp->buttons.config_method.set_method = tp_button_config_click_set_method;
@@ -825,35 +801,54 @@ tp_check_clickfinger_distance(struct tp_dispatch *tp,
 			      struct tp_touch *t2)
 {
 	double x, y;
+	int within_distance = 0;
+	int xres, yres;
+	int bottom_threshold;
 
 	if (!t1 || !t2)
 		return 0;
 
+	if (t1->thumb.state == THUMB_STATE_YES ||
+	    t2->thumb.state == THUMB_STATE_YES)
+		return 0;
+
 	x = abs(t1->point.x - t2->point.x);
 	y = abs(t1->point.y - t2->point.y);
 
-	/* no resolution, so let's assume they're close enough together */
-	if (tp->device->abs.fake_resolution) {
-		int w, h;
+	xres = tp->device->abs.absinfo_x->resolution;
+	yres = tp->device->abs.absinfo_y->resolution;
+	x /= xres;
+	y /= yres;
 
-		/* Use a maximum of 30% of the touchpad width or height if
-		 * we dont' have resolution. */
-		w = tp->device->abs.dimensions.x;
-		h = tp->device->abs.dimensions.y;
+	/* maximum horiz spread is 40mm horiz, 30mm vert, anything wider
+	 * than that is probably a gesture. */
+	if (x > 40 || y > 30)
+		goto out;
 
-		return (x < w * 0.3 && y < h * 0.3) ? 1 : 0;
-	} else {
-		/* maximum spread is 40mm horiz, 20mm vert. Anything wider than that
-		 * is probably a gesture. The y spread is small so we ignore clicks
-		 * with thumbs at the bottom of the touchpad while the pointer
-		 * moving finger is still on the pad */
+	within_distance = 1;
 
-		x /= tp->device->abs.absinfo_x->resolution;
-		y /= tp->device->abs.absinfo_y->resolution;
+	/* if y spread is <= 20mm, they're definitely together. */
+	if (y <= 20)
+		goto out;
 
-		return (x < 40 && y < 20) ? 1 : 0;
-	}
+	/* if they're vertically spread between 20-40mm, they're not
+	 * together if:
+	 * - the touchpad's vertical size is >50mm, anything smaller is
+	 *   unlikely to have a thumb resting on it
+	 * - and one of the touches is in the bottom 20mm of the touchpad
+	 *   and the other one isn't
+	 */
 
+	if (tp->device->abs.dimensions.y/yres < 50)
+		goto out;
+
+	bottom_threshold = tp->device->abs.absinfo_y->maximum - 20 * yres;
+	if ((t1->point.y > bottom_threshold) !=
+		    (t2->point.y > bottom_threshold))
+		within_distance = 0;
+
+out:
+	return within_distance;
 }
 
 static uint32_t
@@ -863,27 +858,24 @@ tp_clickfinger_set_button(struct tp_dispatch *tp)
 	unsigned int nfingers = tp->nfingers_down;
 	struct tp_touch *t;
 	struct tp_touch *first = NULL,
-			*second = NULL,
-			*third = NULL;
-	uint32_t close_touches = 0;
+			*second = NULL;
 
-	if (nfingers < 2 || nfingers > 3)
+	if (nfingers != 2)
 		goto out;
 
-	/* two or three fingers down on the touchpad. Check for distance
+	/* two fingers down on the touchpad. Check for distance
 	 * between the fingers. */
 	tp_for_each_touch(tp, t) {
 		if (t->state != TOUCH_BEGIN && t->state != TOUCH_UPDATE)
 			continue;
 
+		if (t->thumb.state == THUMB_STATE_YES)
+			continue;
+
 		if (!first)
 			first = t;
 		else if (!second)
 			second = t;
-		else if (!third) {
-			third = t;
-			break;
-		}
 	}
 
 	if (!first || !second) {
@@ -891,24 +883,18 @@ tp_clickfinger_set_button(struct tp_dispatch *tp)
 		goto out;
 	}
 
-	close_touches |= tp_check_clickfinger_distance(tp, first, second) << 0;
-	close_touches |= tp_check_clickfinger_distance(tp, second, third) << 1;
-	close_touches |= tp_check_clickfinger_distance(tp, first, third) << 2;
-
-	switch(__builtin_popcount(close_touches)) {
-	case 0: nfingers = 1; break;
-	case 1: nfingers = 2; break;
-	default: nfingers = 3; break;
-	}
+	if (tp_check_clickfinger_distance(tp, first, second))
+		nfingers = 2;
+	else
+		nfingers = 1;
 
 out:
 	switch (nfingers) {
 	case 0:
 	case 1: button = BTN_LEFT; break;
 	case 2: button = BTN_RIGHT; break;
-	case 3: button = BTN_MIDDLE; break;
 	default:
-		button = 0;
+		button = BTN_MIDDLE; break;
 		break;
 	}
 
@@ -927,8 +913,8 @@ tp_notify_clickpadbutton(struct tp_dispatch *tp,
 		struct evdev_dispatch *dispatch = tp->buttons.trackpoint->dispatch;
 		struct input_event event;
 
-		event.time.tv_sec = time/1000;
-		event.time.tv_usec = (time % 1000) * 1000;
+		event.time.tv_sec = time / ms2us(1000);
+		event.time.tv_usec = time % ms2us(1000);
 		event.type = EV_KEY;
 		event.code = button;
 		event.value = (state == LIBINPUT_BUTTON_STATE_PRESSED) ? 1 : 0;
@@ -943,6 +929,10 @@ tp_notify_clickpadbutton(struct tp_dispatch *tp,
 	if (tp->device->suspended)
 		return 0;
 
+	/* A button click always terminates edge scrolling, even if we
+	 * don't end up sending a button event. */
+	tp_edge_scroll_stop_events(tp, time);
+
 	/*
 	 * If the user has requested clickfinger replace the button chosen
 	 * by the softbutton code with one based on the number of fingers.
@@ -969,7 +959,6 @@ tp_post_clickpadbutton_buttons(struct tp_dispatch *tp, uint64_t time)
 
 	current = tp->buttons.state;
 	old = tp->buttons.old_state;
-	button = 0;
 	is_top = 0;
 
 	if (!tp->buttons.click_pending && current == old)
