@@ -32,9 +32,6 @@
 
 #include "evdev-mt-touchpad.h"
 
-#define CASE_RETURN_STRING(a) case a: return #a
-
-#define DEFAULT_SCROLL_LOCK_TIMEOUT 300 /* ms */
 /* Use a reasonably large threshold until locked into scrolling mode, to
    avoid accidentally locking in scrolling mode when trying to use the entire
    touchpad to move the pointer. The user can wait for the timeout to trigger
@@ -92,6 +89,23 @@ tp_touch_get_edge(struct tp_dispatch *tp, struct tp_touch *t)
 	return edge;
 }
 
+static inline void
+tp_edge_scroll_set_timer(struct tp_dispatch *tp,
+			 struct tp_touch *t)
+{
+	const int DEFAULT_SCROLL_LOCK_TIMEOUT = ms2us(300);
+	/* if we use software buttons, we disable timeout-based
+	 * edge scrolling. A finger resting on the button areas is
+	 * likely there to trigger a button event.
+	 */
+	if (tp->buttons.click_method ==
+	    LIBINPUT_CONFIG_CLICK_METHOD_BUTTON_AREAS)
+		return;
+
+	libinput_timer_set(&t->scroll.timer,
+			   t->millis + DEFAULT_SCROLL_LOCK_TIMEOUT);
+}
+
 static void
 tp_edge_scroll_set_state(struct tp_dispatch *tp,
 			 struct tp_touch *t,
@@ -108,8 +122,7 @@ tp_edge_scroll_set_state(struct tp_dispatch *tp,
 	case EDGE_SCROLL_TOUCH_STATE_EDGE_NEW:
 		t->scroll.edge = tp_touch_get_edge(tp, t);
 		t->scroll.initial = t->point;
-		libinput_timer_set(&t->scroll.timer,
-				   t->millis + DEFAULT_SCROLL_LOCK_TIMEOUT);
+		tp_edge_scroll_set_timer(tp, t);
 		break;
 	case EDGE_SCROLL_TOUCH_STATE_EDGE:
 		break;
@@ -272,31 +285,11 @@ int
 tp_edge_scroll_init(struct tp_dispatch *tp, struct evdev_device *device)
 {
 	struct tp_touch *t;
-	int width, height;
 	int edge_width, edge_height;
 
-	width = device->abs.dimensions.x;
-	height = device->abs.dimensions.y;
-
-	switch (tp->model) {
-	case MODEL_ALPS:
-		edge_width = width * .15;
-		edge_height = height * .15;
-		break;
-	case MODEL_APPLETOUCH: /* unibody are all clickpads, so N/A */
-		edge_width = width * .085;
-		edge_height = height * .085;
-		break;
-	default:
-		/* For elantech and synaptics, note for lenovo #40 series,
-		 * e.g. the T440s min/max are the absolute edges, not the
-		 * recommended ones as usual with synaptics. But these are
-		 * clickpads, so N/A.
-		 */
-		edge_width = width * .04;
-		edge_height = height * .054;
-		break;
-	}
+	/* 7mm edge size */
+	edge_width = device->abs.absinfo_x->resolution * 7;
+	edge_height = device->abs.absinfo_y->resolution * 7;
 
 	tp->scroll.right_edge = device->abs.absinfo_x->maximum - edge_width;
 	tp->scroll.bottom_edge = device->abs.absinfo_y->maximum - edge_height;
@@ -349,7 +342,7 @@ tp_edge_scroll_handle_state(struct tp_dispatch *tp, uint64_t time)
 int
 tp_edge_scroll_post_events(struct tp_dispatch *tp, uint64_t time)
 {
-	struct libinput_device *device = &tp->device->base;
+	struct evdev_device *device = tp->device;
 	struct tp_touch *t;
 	enum libinput_pointer_axis axis;
 	double *delta;
@@ -367,11 +360,16 @@ tp_edge_scroll_post_events(struct tp_dispatch *tp, uint64_t time)
 		if (t->palm.state != PALM_NONE)
 			continue;
 
+		/* only scroll with the finger in the previous edge */
+		if (t->scroll.edge &&
+		    (tp_touch_get_edge(tp, t) & t->scroll.edge) == 0)
+			continue;
+
 		switch (t->scroll.edge) {
 			case EDGE_NONE:
 				if (t->scroll.direction != -1) {
 					/* Send stop scroll event */
-					pointer_notify_axis(device, time,
+					evdev_notify_axis(device, time,
 						AS_MASK(t->scroll.direction),
 						LIBINPUT_POINTER_AXIS_SOURCE_FINGER,
 						&zero,
@@ -392,12 +390,13 @@ tp_edge_scroll_post_events(struct tp_dispatch *tp, uint64_t time)
 		}
 
 		normalized = tp_get_delta(t);
-		normalized = tp_filter_motion(tp, &normalized, time);
+		/* scroll is not accelerated */
+		normalized = tp_filter_motion_unaccelerated(tp, &normalized, time);
 
 		switch (t->scroll.edge_state) {
 		case EDGE_SCROLL_TOUCH_STATE_NONE:
 		case EDGE_SCROLL_TOUCH_STATE_AREA:
-			log_bug_libinput(device->seat->libinput,
+			log_bug_libinput(tp_libinput_context(tp),
 					 "unexpected scroll state %d\n",
 					 t->scroll.edge_state);
 			break;
@@ -418,11 +417,11 @@ tp_edge_scroll_post_events(struct tp_dispatch *tp, uint64_t time)
 		if (*delta == 0.0)
 			continue;
 
-		pointer_notify_axis(device, time,
-				    AS_MASK(axis),
-				    LIBINPUT_POINTER_AXIS_SOURCE_FINGER,
-				    &normalized,
-				    &zero_discrete);
+		evdev_notify_axis(device, time,
+				  AS_MASK(axis),
+				  LIBINPUT_POINTER_AXIS_SOURCE_FINGER,
+				  &normalized,
+				  &zero_discrete);
 		t->scroll.direction = axis;
 
 		tp_edge_scroll_handle_event(tp, t, SCROLL_EVENT_POSTED);
@@ -434,14 +433,14 @@ tp_edge_scroll_post_events(struct tp_dispatch *tp, uint64_t time)
 void
 tp_edge_scroll_stop_events(struct tp_dispatch *tp, uint64_t time)
 {
-	struct libinput_device *device = &tp->device->base;
+	struct evdev_device *device = tp->device;
 	struct tp_touch *t;
 	const struct normalized_coords zero = { 0.0, 0.0 };
 	const struct discrete_coords zero_discrete = { 0.0, 0.0 };
 
 	tp_for_each_touch(tp, t) {
 		if (t->scroll.direction != -1) {
-			pointer_notify_axis(device, time,
+			evdev_notify_axis(device, time,
 					    AS_MASK(t->scroll.direction),
 					    LIBINPUT_POINTER_AXIS_SOURCE_FINGER,
 					    &zero,
