@@ -956,7 +956,7 @@ litest_copy_file(const char *dest, const char *src, const char *header)
 	in = open(src, O_RDONLY);
 	litest_assert_int_gt(in, -1);
 	/* lazy, just check for error and empty file copy */
-	litest_assert_int_gt(sendfile(out, in, NULL, 4096), 0);
+	litest_assert_int_gt(sendfile(out, in, NULL, 40960), 0);
 	close(out);
 	close(in);
 }
@@ -1255,7 +1255,8 @@ litest_event(struct litest_device *d, unsigned int type,
 }
 
 static bool
-axis_replacement_value(struct axis_replacement *axes,
+axis_replacement_value(struct litest_device *d,
+		       struct axis_replacement *axes,
 		       int32_t evcode,
 		       int32_t *value)
 {
@@ -1266,7 +1267,7 @@ axis_replacement_value(struct axis_replacement *axes,
 
 	while (axis->evcode != -1) {
 		if (axis->evcode == evcode) {
-			*value = axis->value;
+			*value = litest_scale(d, evcode, axis->value);
 			return true;
 		}
 		axis++;
@@ -1307,7 +1308,7 @@ litest_auto_assign_value(struct litest_device *d,
 		value = touching ? 0 : 1;
 		break;
 	default:
-		if (!axis_replacement_value(axes, ev->code, &value) &&
+		if (!axis_replacement_value(d, axes, ev->code, &value) &&
 		    d->interface->get_axis_default)
 			d->interface->get_axis_default(d, ev->code, &value);
 		break;
@@ -1671,17 +1672,36 @@ litest_keyboard_key(struct litest_device *d, unsigned int key, bool is_press)
 	litest_button_click(d, key, is_press);
 }
 
+static int
+litest_scale_axis(const struct litest_device *d,
+		  unsigned int axis,
+		  double val)
+{
+	const struct input_absinfo *abs;
+
+	litest_assert_double_ge(val, 0.0);
+	litest_assert_double_le(val, 100.0);
+
+	abs = libevdev_get_abs_info(d->evdev, axis);
+	litest_assert_notnull(abs);
+
+	return (abs->maximum - abs->minimum) * val/100.0 + abs->minimum;
+}
+
 int
 litest_scale(const struct litest_device *d, unsigned int axis, double val)
 {
 	int min, max;
-	litest_assert_int_ge((int)val, 0);
-	litest_assert_int_le((int)val, 100);
-	litest_assert_int_le(axis, (unsigned int)ABS_Y);
+	litest_assert_double_ge(val, 0.0);
+	litest_assert_double_le(val, 100.0);
 
-	min = d->interface->min[axis];
-	max = d->interface->max[axis];
-	return (max - min) * val/100.0 + min;
+	if (axis <= ABS_Y) {
+		min = d->interface->min[axis];
+		max = d->interface->max[axis];
+		return (max - min) * val/100.0 + min;
+	} else {
+		return litest_scale_axis(d, axis, val);
+	}
 }
 
 void
